#!/usr/bin/env python
#
# Copyright 2018-2019 Axel Huebl
#
# License: CC-BY-SA 4.0 International
#
import matplotlib as mpl
#mpl.use('Agg') # non-interactive backend
import matplotlib.pyplot as plt
from matplotlib.colors import colorConverter, LogNorm, Normalize
from picongpu.plugins.data import EnergyHistogramData
import h5py as h5
import numpy as np


# a0=16  # 16, 30, 42

# sfilter='fwRCF'    # 'fwRCF', 'fwPinhole', 'all', 'originFront', 'originRear'
geom='mixed_30nc'  # mixed_30nc separ_30nc  separ_300nc separ_1000nc separ_1700nc

step_peak_target = 8190  # 64.3 fs
dt = 3.33e-9 / ( 1.415 * 2.99792458e8 )  # 7.84995e-18  # s
tau = 30.e-15  # s


def getData(species, sfilter, a0, geom, dratio, suffix=""):
    """..."""
    #eneHist = EnergyHistogramData("a0_" + str(a0) + "_nofmath/2D_" + geom + "_dratio" + dratio + suffix)
    eneHist = EnergyHistogramData("a0_" + str(a0) + "/2D_" + geom + "_dratio" + dratio + suffix)
    its = eneHist.get_iterations(species, sfilter)
    counts, bins = eneHist.get(species, sfilter, its)
    if species == 'D':
        bins = bins * 0.5
    return counts, bins, its

def plotAndSave(species, sfilter, a0, geom, dratios, suffix=""):
    """..."""
    for dratio in dratios:
        if species == "H" and dratio == "1.00":
            continue
        if species == "D" and dratio == "0.00":
            continue

        counts_dict, bins, its = getData(species, sfilter, a0, geom, dratio, suffix)
        counts = np.array([counts_dict[i] for i in counts_dict.keys()])

        f = plt.figure(figsize=[4, 3])
        ax = f.add_axes([0.13, 0.17, 0.65, 0.65])
        ax2 = ax.twiny()
        cax = f.add_axes([0.79, 0.17, 0.05, 0.65])

        im = ax.imshow(
            counts.T,
            extent = [
                np.min(its) * dt * 1.e12,
                np.max(its) * dt * 1.e12,
                np.min(bins) * 1.e-3,
                np.max(bins) * 1.e-3
            ],
            interpolation = 'nearest',
            aspect = 'auto',
            origin='lower'
            ,norm = LogNorm()
            #,cmap=myCmaps[species]
            # MPL bug: https://github.com/matplotlib/matplotlib/issues/10051
            ,vmin=np.max(counts)*1.e-9
            ,vmax=np.max(counts)
        )
        t_peak = step_peak_target * dt
        ax.axvline(
            x=(t_peak - 2. * tau) * 1.e12,
            linestyle=':',
            color='red')
        ax.axvline(
            x=(t_peak + 2. * tau) * 1.e12,
            linestyle=':',
            color='red')

        # annotations
        cbar = f.colorbar(im, cax=cax)
        cbar.set_label(r'count  [arb. u.]')

        if species == "e":
            ax.set_ylabel(r'$K$ for {} ({}) [MeV]'.format(
            species, sfilter))
        else:
            ax.set_ylabel(r'$K / A$ for {} ({}) [MeV]'.format(
            species, sfilter))

        # time steps
        ax2.set_xlim([np.min(its), np.max(its)])
        # ax2.set_yticklabels([])

        ax.set_xlabel(r'time  [ps]')
        ax2.set_xlabel(r'time  [steps]')

        #plt.title("{0} ({1})".format(species, sfilter))
        #plt.legend()

        #plt.show()
        plt.savefig(geom + "/energyEvolution/" +
            "a0_" + str(a0) + "_" +
            #"nofmath_" +
            species + "_" + sfilter + "_" +
            geom +
            "_dratio" + dratio +
            "_energyEvolution.svg", dpi=300)
        plt.close()

all_dratios = [
    "0.00", "0.10", "0.20", "0.30", "0.40", "0.50",
    "0.60", "0.70", "0.80", "0.90", "0.98", "1.00"
]

#for species in ['H', 'D']:
#    plotAndSave(species, sfilter, a0, geom, dratios=["0.00", "0.10", "0.50", "0.90", "0.98", "1.00"])

for geom in ['mixed_30nc']:  # 'separ_30nc'
    for a0 in [8, 16, 23, 30, 42]:
        for species in ['e', 'H', 'D']:
            for sfilter in ['all', 'fwPinhole', 'fwRCF', 'originFront', 'originRear']:
                plotAndSave(species, sfilter, a0, geom, dratios=all_dratios)
